// Node.cpp: implementation of the Node class.
//
//////////////////////////////////////////////////////////////////////

#include "Node.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Node::Node(string data_arg, NodePtr prev_arg, NodePtr next_arg)
{
	data = data_arg;
	next = next_arg;
	prev = prev_arg;
}

Node::Node()
{
}

void Node::makeHeadAndTail (NodePtr &head, NodePtr &tail)
{
	head = new Node("",NULL,NULL);
	tail = new Node("",NULL,NULL);
	head->next = tail;
	tail->prev = head;
}

Node::~Node()
{

}

string Node::getData () const
{
	return data;
}

NodePtr Node::getNext () const {
	return next;
}

NodePtr Node::getPrev () const {
	return prev;
}

bool Node::isHead () const
{
	return prev == NULL;
}

bool Node::isTail () const
{
	return next == NULL;
}

NodePtr Node::stepForward (int howMany) 
{
	if (howMany == 0)
		return this;
	else
		return next->stepForward (howMany - 1);
}

NodePtr Node::stepBackward (int howMany) 
{
	if (howMany == 0)
		return this;
	else
		return prev->stepBackward (howMany - 1);
}

void Node::addAfter (string s)
{
	this->next = this->next->prev = new Node(s, this, this->next);
}

void Node::addBefore (string s)
{
	this->prev = this->prev->next = new Node(s, this->prev, this);
}

void Node::remove ()
{
	this->next->prev = this->prev;
	this->prev->next = this->next;
	delete this;
}

int Node::countNodesAfter () const
{
	if (this->next == NULL) // we're at the tail
		return -1;
	else
		return 1 + this->next->countNodesAfter();

	// Why -1, you ask, rather than 0?  Well, the last "real" node (the one
	// with 0 nodes after it) is the one that POINTS to the tail.  Once we
	// get to the tail, we've gone one too far: not only are there no nodes
	// after this, but even this itself isn't a proper node.
}

int Node::countNodesBefore () const
{
	if (this->prev == NULL) // we're at the head
		return -1;
	else
		return 1 + this->prev->countNodesBefore();
	// See above.
}
