// List.cpp: implementation of the List class.
//
//////////////////////////////////////////////////////////////////////

#include "List.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

List::List()
{
	Node::makeHeadAndTail(head, tail);
}


List::~List()
{
	
}


int List::length() const
{
	int answer = 0;
	NodePtr current = head;
	while (! current->getNext()->isTail())
	{
		current = current->getNext();
		answer++;
	}
	return answer;
}



int List::recursive_length() const
{
	return head->countNodesAfter();
	// or, equivalently, tail->countNodesBefore()
}


std::string descr_helper(NodePtr current)
{
	if (current->isTail())
		return "";
	else if (current->getNext()->isTail())
		return current->getData();
	else
		return current->getData() + "," +
			descr_helper(current->getNext());
}

std::string List::description() const
{
	return "List [" + descr_helper(head->getNext()) + "]";
}

std::string List::iterative_description() const
{
	std::string answer = "List [";
	for (NodePtr current = head->getNext();
		 ! current->isTail();
		 current = current->getNext())
	{
		answer += current->getData();
		if (! current->getNext()->isTail())
			answer += ",";
	}
	return answer + "]";
}

void List::addToFront(string s)
{
	head->addAfter(s);
}

void List::addToBack(string s)
{
	tail->addBefore(s);
}


void List::addAtPosition(string s, int where)
{
	head->stepForward(where)->addAfter(s);
}

void List::removeFirst()
{
	head->stepForward(1)->remove();
}

void List::removeLast()
{
	tail->stepBackward(1)->remove();
}

void List::removeAtPosition (int where)
{
	head->stepForward(where+1)->remove();
}

List * List::map(string (*f)(string))
{
	List *result = new List();
	for (NodePtr current = head->getNext();
		 ! current->isTail();
		 current = current->getNext())
		 {
			string s = current->getData();
			result->addToBack(f(s));
		 }
	return result;
}